/*
 * RemoveAction.cpp
 *
 *  Created on: May 9, 2010
 *      Author: heber
 */

#include "Helpers/MemDebug.hpp"

#include "Actions/AtomAction/RemoveAction.hpp"
#include "Actions/ActionRegistry.hpp"
#include "atom.hpp"
#include "AtomicInfo.hpp"
#include "Descriptors/AtomDescriptor.hpp"
#include "Helpers/Log.hpp"
#include "molecule.hpp"
#include "Helpers/Verbose.hpp"
#include "World.hpp"

#include <iostream>
#include <string>

using namespace std;

#include "UIElements/UIFactory.hpp"
#include "UIElements/Dialog.hpp"
#include "Actions/ValueStorage.hpp"

// memento to remember the state when undoing

class AtomRemoveState : public ActionState {
public:
  AtomRemoveState(std::vector<AtomicInfo> _Walkers) :
    Walkers(_Walkers)
  {}
  std::vector<AtomicInfo> Walkers;
};

const char AtomRemoveAction::NAME[] = "remove-atom";

AtomRemoveAction::AtomRemoveAction() :
  Action(NAME)
{}

AtomRemoveAction::~AtomRemoveAction()
{}

void AtomRemove() {
  ActionRegistry::getInstance().getActionByName(AtomRemoveAction::NAME)->call(Action::NonInteractive);
};

Dialog* AtomRemoveAction::fillDialog(Dialog *dialog) {
  ASSERT(dialog,"No Dialog given when filling action dialog");

  dialog->queryEmpty(NAME, ValueStorage::getInstance().getDescription(NAME));

  return dialog;
}

Action::state_ptr AtomRemoveAction::performCall() {
  atom *first = NULL;

  // create undo state
  std::vector<AtomicInfo> Walkers;
  for (World::AtomSelectionIterator iter = World::getInstance().beginAtomSelection(); iter != World::getInstance().endAtomSelection(); ++iter) {
    Walkers.push_back(AtomicInfo(*(iter->second)));
  }
  AtomRemoveState *UndoState = new AtomRemoveState(Walkers);

  // remove all selected atoms
//  std::vector<molecule *> molecules = World::getInstance().getAllMolecules();
  for (World::AtomSelectionIterator iter = World::getInstance().beginAtomSelection(); iter != World::getInstance().endAtomSelection(); ++iter) {
    first = iter->second;
    DoLog(1) && (Log() << Verbose(1) << "Removing atom " << first->getId() << "." << endl);
//    // TODO: this is not necessary when atoms and their storing to file are handled by the World
//    // simply try to erase in every molecule found
//    for (std::vector<molecule *>::iterator iter = molecules.begin();iter != molecules.end(); ++iter) {
//      (*iter)->erase(first);
//    }
    World::getInstance().destroyAtom(first);
  }
  return Action::state_ptr(UndoState);
}

Action::state_ptr AtomRemoveAction::performUndo(Action::state_ptr _state) {
  AtomRemoveState *state = assert_cast<AtomRemoveState*>(_state.get());

  size_t i=0;
  for (; i<state->Walkers.size(); ++i) {
    // re-create the atom
    DoLog(1) && (Log() << Verbose(1) << "Re-adding atom " << state->Walkers[i].getId() << "." << endl);
    atom *Walker = World::getInstance().createAtom();
    if (!state->Walkers[i].setAtom(*Walker)) {
      DoeLog(1) && (eLog() << Verbose(1) << "Failed to set id." << endl);
      World::getInstance().destroyAtom(Walker);
      break;
    }
  }
  if (i<state->Walkers.size()) {
    // remove all previous ones, too
    for (size_t j=0;j<i;++j)
      World::getInstance().destroyAtom(state->Walkers[j].getId());
    // and announce the failure of the undo
    return Action::failure;
  }
  return Action::state_ptr(_state);
}

Action::state_ptr AtomRemoveAction::performRedo(Action::state_ptr _state){
  AtomRemoveState *state = assert_cast<AtomRemoveState*>(_state.get());

  // simple remove again all previously added atoms
  for (size_t i=0; i<state->Walkers.size(); ++i) {
    DoLog(1) && (Log() << Verbose(1) << "Re-removing atom " << state->Walkers[i].getId() << "." << endl);
    World::getInstance().destroyAtom(state->Walkers[i].getId());
  }

  return Action::state_ptr(_state);
}

bool AtomRemoveAction::canUndo() {
  return true;
}

bool AtomRemoveAction::shouldUndo() {
  return true;
}

const string AtomRemoveAction::getName() {
  return NAME;
}
